# frozen_string_literal: true

require 'json'

module MMMD
  # Utils for working with entities in strings
  module EntityUtils
    ENTITY_DATA = JSON.parse(File.read("#{__dir__}/entities.json"))

    # Decode html entities in string
    # @param string [String]
    # @return [String]
    def self.decode_entities(string)
      string = string.gsub(/&#\d{1,7};/) do |match|
        match[1..-2].to_i.chr("UTF-8")
      end
      string = string.gsub(/&#[xX][\dA-Fa-f]{1,6};/) do |match|
        match[3..-2].to_i(16).chr("UTF-8")
      end
      string.gsub(/&\w+;/) do |match|
        ENTITY_DATA[match] ? ENTITY_DATA[match]["characters"] : match
      end
    end

    # Encode unsafe html entities in string (ASCII-compatible)
    # @param string [String]
    # @return [String]
    # @sg-ignore
    def self.encode_entities_ascii(string)
      string.gsub("&", "&amp;")
            .gsub("<", "&lt;")
            .gsub(">", "&gt;")
            .gsub('"', "&quot;")
            .gsub("'", "&#39;")
            .gsub(/[^\x00-\x7F]/) do |match|
              "&#x#{match.codepoints[0]};"
            end
    end

    # Encode unsafe html entities in string
    # @param string [String]
    # @return [String]
    # @sg-ignore
    def self.encode_entities(string)
      string.gsub("&", "&amp;")
            .gsub("<", "&lt;")
            .gsub(">", "&gt;")
            .gsub('"', "&quot;")
            .gsub("'", "&#39;")
    end

    # Encode uri components that may break HTML syntax
    # @param string [String]
    # @return [String]
    def self.encode_uri(string)
      string.gsub('"', "%22")
            .gsub("'", "%27")
            .gsub(" ", "%20")
    end
  end
end
